# Dependency

Dependency is something your program depends on -- dependencies are [bad](shit.md)! Unfortunately they are also unavoidable. We at least try to minimize dependencies as much as possible while keeping our program functioning as intended, and those we can't avoid we try to abstract in order to be able to quickly drop-in replace them with alternatives.

Having many dependencies is a sign of [bloat](bloat.md) and bad design. Unfortunately this is the reality of mainstream programming. For example at the time of writing this [Chromium](chromium.md) in [Debian](debian.md) requires (recursively) 395 packages LMAO xD And these are just runtime dependencies...

In [software](software.md) development context we usually talk about software dependencies, typically [libraries](library.md) and other software [packages](package.md). However, there are many other types of dependencies we need to consider when striving for the best programs. Let us list just some of the possible types:

- [software](software.md)
  - [libraries](library.md)
  - [compiler](compiler.md) supporting specific language standard
  - [build system](build_system.md)
  - [GUI](gui.md) capability
  - [operating system](operating_system.md) and its services such as presence of a [window manager](file_system.md), [desktop environment](desktop_environment.md), presence of a [file system](file_system.md) etc.
  - [Internet](internet.md) connection
- [hardware](hardware.md)
   - [computing resources](computing_resources.md) (sufficient RAM, CPU speed, ...)
  - [graphics card](gpu.md)
  - [floating point unit](fpu.md) and other [coprocessors](coprocessor.md)
  - CPU features such as special instructions
  - [mouse](mouse.md), [speakers](monitor.md) and other I/O devices
- other:
  - know-how/education: Your program may require specific knowledge, e.g. knowledge of advanced math to be able to meaningfully modify the program, or nonnegligiable amount of time spent studying your codebase.
  - running cost: e.g. electricity, Internet connection cost
  - culture: Your program may require the culture to allow what it is presenting or dealing with.
  
Good program will take into account all kinds of these dependencies and try to minimize them to offer freedom, stability and safety while keeping its functionality or reducing it only very little.

Why are dependencies so bad? Some of the reasons are:

- **less secure** (more [attack surface](attack_surface.md), i.e. potential for vulnerabilities which may arise in the dependencies)
- **more buggy** (more [fuck up surface](fuck_up_surface.md))
- **more expensive to [maintain](maintenance.md)** (requires someone's constant attention to just keep the dependencies up to date and keeping up with their changing API)
- **less [future proof](future_proof.md)** and more **fragile** (your program dies as soon as one of its dependencies, or any dependency of these dependencies)
- **less under your control** (in practice it's extremely difficult to modify and maintain a library you use even if it's [free](free_software.md), so you're typically doomed to just accept whatever it does)
- **more dangerous [legally](law.md)** (reusing work of other people requires dealing with several to many different licenses with possibly wild conditions and there's always a chance of someone starting to make trouble such as threatening to withdraw a license)

## How to Avoid Them

TODO